//***********************************************
//***********************************************
//****   TEMPLATE FÜR W-LAN und JUMPER       ****
//***********************************************
//***********************************************

//Telnetverbindung:
//    sudo telnet "10.0.0.11" "8081"

//   avr-g++ -O2 -mmcu=atmega32 sumobot.cpp -o sumobot.elf 
//   avr-objcopy -O ihex -j .text -j .data sumobot.elf sumobot.hex 

//   Upload über RobotLoader_20100712

//Neu bei TEMPLATE2:
//auch gepuffertes Lesen implementiert.
//Neu Initialisierung für W-LAN-Modul möglich!

//Einbau zwei weiterer Zeitebenen.
//Erstes Beispiel eines einfachen Reflexagenten

// NEU NEU NEU NEU NEU NEU NEU NEU NEU NEU NEU
//
//   Verwendete Anschlüsse für die JUMPER:
//
//   Stiftleiste I2C:
//   INT1 SCL SDA GND  ACHTUNG INT1 beim R6V2 ist PA4 !!!!!
//   INT1 == PA4  als Ausgang auf GND setzen JUMPER0
//   SCL  == PC0                             JUMPER0 Eingang mit Pullup
//   SDA  == PC1                             JUMPER1 Eingang mit Pullup
//   GND                                     JUMPER1
//   VDD  nicht verbunden VORSICHT MIT JUMPERN!!!!
//
//   Konfiguration und Benutzung in JUMPER.h, bzw. Klasse Jumper, Objekt jumper
//
//
// NEU NEU NEU NEU NEU NEU NEU NEU NEU NEU NEU

#include<avr/io.h>
#include<avr/interrupt.h>

//10000Hz
//dt = 0.0001s
#define MOTORMAX 799

volatile unsigned int ZEITSCHRITTE = 0;
volatile unsigned long SEKUNDEN = 0;
volatile int ZUSTAND0=2; //0==Leerer Loop, 1==Ton 2500Hz, 2==Sensoren aktualisieren, 3==Seriell aktualisieren
volatile int ZUSTAND1=0;
volatile int ZUSTAND2=1;  //0==leer, 1==einfacher Reflexagent
volatile int ZUSTAND3=1;  //0==leer, 1==Seriell aktualisieren
volatile int ZUSTAND4=1;  //0==leer, 1==Sensorzustände zeigen UND Jumper UND zuletzt empf. Zeichen, 2==Jumperzustände zeigen
volatile int ZUSTAND5=0; 
volatile int ZUSTAND6=0;  //0==leer, 1==WLAN Initialisierung , 2==WLAN2 Initialisierung 

//Zustände definieren:
bool AMRAND = false;
bool AMGEGNER = false;

#include "LED.h"
#include "TIMER1.h"
#include "SENSOR.h"
#include "SERIAL.h"
#include "JUMPER.h"
#include "WLAN.h"
#include "LOOP.h"

//So können höhere Ebenen niedrigere konfigurieren:

#include "LOOP0.h"
LOOP* loops0[4] = {&loop0_LEER,&loop0_Ton2500Hz,&loop0_Sensor,&loop0_Serial_aktualisieren};

#include "LOOP1.h"
LOOP* loops1[1] = {&loop1_LEER};

#include "LOOP2.h"
LOOP* loops2[3] = {&loop2_LEER,&loop2_EINFACHER_REFLEXAGENT,&loop2_FERNGESTEUERT};

#include "LOOP3.h"
LOOP* loops3[2] = {&loop3_LEER,&loop3_Serial_aktualisieren};

#include "LOOP4.h"
LOOP* loops4[3] = {&loop4_LEER,&loop4_Zeige_Sensorwerte,&loop4_Zeige_Jumperzustand};

#include "LOOP5.h"
LOOP* loops5[1] = {&loop5_LEER};

#include "LOOP6.h"
LOOP* loops6[3] = {&loop6_LEER,&loop6_WLAN,&loop6_WLAN2};

unsigned char LEVEL0=0,LEVEL1=0,LEVEL2=0,LEVEL3=0,LEVEL4=0,LEVEL5=0,LEVEL6=0;

//Idee: Zeitliche Auflösung der verschiedenen Abstraktionslevels:
//        0       1       2       3
//        01234567890123456789012345678901
//        01234567012345670123456701234567
//LEVEL0: 01010101010101010101010101010101 z%2==1     128 Schritte == z0..256-1   5000Hz-Schritte    dt=0,0002s T=0,0256s
//LEVEL1: 10001000100010001000100010001000 z%4==0     128 Schritte == z0..512-1   2500Hz-Schritte    dt=0,0004s T=0,0512s
//LEVEL2: 00100000001000000010000000100000 z%8==2     128 Schritte == z0..1024-1  1250Hz-Schritte    dt=0,0008s T=0,1024s
//LEVEL3: 00000010000000000000001000000000 z%16==6    128 Schritte == z0..2048-1   625Hz-Schritte    dt=0,0016s T=0,2048s
//LEVEL4: 00000000000000100000000000000000 z%32==14   128 Schritte == z0..4096-1   312,25Hz-Schritte dt=0,0032s T=0,4096s

//LEVEL5: 00000000000000000000000000000010 z%64==30   128 Schritte == z0..8192-1   156,125Hz-Schritte dt=0,0064s T=0,8192s
//LEVEL6: 00000000000000000000000000000010 z%128==62  128 Schritte == z0..16384-1  78,0625Hz-Schritte dt=0,0128s T=1,6384s

//Systematischer:
//0123456701234567
//1010101010101010 z%2 == 0
//0100010001000100 z%4 == 1  // ==2-1
//0001000000010000 z%8 == 3  // ==4-1
//0000000100000000 z%16== 7  // ==8-1 (immer vorangehende 2er-Potenz minus 1
//
//  LEVEL 0, 1, 2, ...
//  LEVEL x =>  z%2**(x+1) == 2**x-1
//  
//  Also:
//  LEVEL0: z%2  ==0
//  LEVEL1: z%4  ==1
//  LEVEL2: z%8  ==3
//  LEVEL3: z%16 ==7
//  LEVEL4: z%32 ==15
//  LEVEL5: z%64 ==31
//  LEVEL6: z%128==63


ISR(TIMER1_COMPA_vect)  //Ein Durchgang
{
    cli(); //Interrupts deaktivieren
    if(ZEITSCHRITTE%2==0)
    {
        loops0[ZUSTAND0]->exec(LEVEL0); 
        LEVEL0++;
        LEVEL0%=128;
    }
    else if(ZEITSCHRITTE%4==1)
    {
        loops1[ZUSTAND1]->exec(LEVEL1); 
        LEVEL1++;
        LEVEL1%=128;
    }
    else if(ZEITSCHRITTE%8==3)
    {
        loops2[ZUSTAND2]->exec(LEVEL2); 
        LEVEL2++;
        LEVEL2%=128;
    }
    else if(ZEITSCHRITTE%16==7)
    {
        loops3[ZUSTAND3]->exec(LEVEL3); 
        LEVEL3++;
        LEVEL3%=128;
    }
    else if(ZEITSCHRITTE%32==15)
    {
        loops4[ZUSTAND4]->exec(LEVEL4); 
        LEVEL4++;
        LEVEL4%=128;
    }
    else if(ZEITSCHRITTE%64==31)
    {
        loops5[ZUSTAND5]->exec(LEVEL5); 
        LEVEL5++;
        LEVEL5%=128;
    }
    else if(ZEITSCHRITTE%128==63)
    {
        loops6[ZUSTAND6]->exec(LEVEL6); 
        LEVEL6++;
        LEVEL6%=128;
    }

    ZEITSCHRITTE++;
    ZEITSCHRITTE%=4096;
    sei(); //Interrupts aktivieren
}


int main()
{
    //Einfach hier eine langsame Fahrt einstellen, um die
    //Stromsensoren beobachten zu können, z.B. beim
    //Blockieren der Ketten:

    //timer1.fahrt(200,200);

    while(1) //Ohne diese Schleife bricht das ganze Programm ab!
    {
    }
}